
# q-expansion of newform 9306.2.a.be, downloaded from the LMFDB on 10 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 9306
weight = 2
poly_data = [-16, 10, 16, -10, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0], 1], [[18, -4, -12, 0, 1], 2], [[-13, -5, 11, 1, -1], 1], [[62, 14, -54, -4, 5], 2]]

hecke_ring_character_values = None
aps_data = [[-1, 0, 0, 0, 0], [0, 0, 0, 0, 0], [0, -1, 0, 0, 0], [1, 0, 0, 1, 0], [-1, 0, 0, 0, 0], [1, 0, -1, 1, 1], [-1, -1, 0, 0, 1], [2, 0, 1, 0, 1], [-1, 0, 0, 2, 1], [0, 1, -1, 1, 2], [2, 0, 0, 0, 0], [0, -1, 2, 0, 0], [-2, 3, 0, 2, 0], [3, 0, -3, 1, 1], [-1, 0, 0, 0, 0], [-1, 1, 0, 2, -1], [-1, -3, -1, 0, 2], [4, -2, 0, -1, -1], [1, -1, 1, -2, -2], [-4, 2, 4, 0, 0], [1, -3, 2, -2, -1], [-1, 2, 1, -1, 1], [-5, 4, 0, 3, 0], [1, -1, 1, -3, -1], [6, -2, 0, -3, -1], [2, 3, 0, 2, -2], [4, -1, 0, -1, -3], [-3, -2, -4, 2, -1], [4, -1, 2, -2, -2], [-2, 3, 1, -2, -3], [6, 0, 0, 4, 4], [-6, 2, 3, 0, 1], [-2, 0, -4, 3, -1], [4, -2, -2, 0, -4], [0, 2, 6, -4, 0], [-3, 5, 1, 2, 2], [3, -3, 5, -3, 1], [6, 2, 2, 2, 0], [6, 0, -3, 0, -1], [-5, 1, 7, -1, 1], [-5, 0, 6, -5, 2], [1, 5, 0, 2, -3], [3, -6, -1, 1, -3], [3, -2, 1, 2, 0], [-1, -2, -3, 3, 3], [8, -4, 3, -4, 1], [-5, 0, 2, -5, 0], [1, -2, 1, -5, -3], [-7, -2, 2, 0, 3], [6, 3, -3, 8, 1], [11, -1, -1, 1, 1], [4, 2, 5, -3, 0], [2, 9, -2, 0, -2], [0, 2, -3, -1, -2], [4, -5, 1, -3, 4], [-7, -5, -5, 4, 0], [5, -2, 3, -10, -4], [-1, 0, 1, -1, 1], [-10, 5, 3, 3, 0], [9, -1, -3, 1, -3], [-1, -4, 2, -6, 3], [-2, 5, 5, -2, -3], [7, 0, 0, 0, 3], [6, 2, -4, -2, 0], [3, -5, -1, -3, -1], [4, 4, 6, -2, 0], [-6, 0, -1, 2, 7], [-17, 5, 1, -1, 3], [-5, 4, 2, 1, 4], [-13, 4, 5, -3, 1], [3, -6, 5, -5, -1], [-1, 2, 0, -7, -2], [-1, -2, -1, 1, 7], [-8, 10, -4, 6, 0], [10, 0, 0, -6, -4], [2, -2, -1, -2, -5], [-1, 0, 1, 5, 3], [-13, 2, 5, -1, -1], [11, 2, 3, -1, 1], [-10, 9, -3, 6, 7], [5, 4, -1, -3, 3], [-9, -6, 1, -4, 2], [20, 0, -1, 5, 2], [-5, -2, -5, 3, 5], [3, 2, 3, -1, -5], [23, -10, 2, -7, -4], [4, -1, 1, -5, 2], [-5, 5, -1, 5, 3], [-8, 2, -2, 9, 1], [-7, 10, -4, 5, 2], [3, 6, -5, -1, -1], [2, -6, 3, -1, -6], [-2, -7, -2, -5, 1], [2, 6, 0, 0, -2], [1, -5, -1, 2, -4], [3, 0, -3, 9, 11], [-3, 2, -5, -3, -5], [13, 7, -3, 3, 1], [16, 4, 0, 6, 2], [-8, 11, -1, 4, -3], [3, 2, -13, 11, 1], [4, 5, -3, 10, 7], [0, 5, 12, -7, -1], [15, 8, 3, -1, -5], [-12, 6, 10, 4, -2], [13, -5, 3, 1, 9], [4, -2, 4, 0, 4], [-12, 0, 2, -2, 4], [9, 4, 4, -2, -1], [-12, -1, 0, 2, 4], [-3, -7, 9, -4, 0], [-2, -5, -5, 0, -5], [6, 6, -8, 6, 2], [-9, -2, 3, -3, 1], [-8, -8, -7, 2, 1], [8, -1, -1, -2, 1], [-5, 6, 4, 11, 4], [15, -6, -1, -7, -7], [23, -1, -4, -2, -1], [4, -6, 2, 0, -6], [6, 10, -2, 4, -2], [-6, 5, 1, -2, 3], [-20, 5, -3, 11, 2], [1, 9, -1, 6, -2], [20, -10, 0, -6, -4], [22, -8, 0, -9, -5], [-5, -8, 3, 7, 1], [-4, 4, 16, -6, 2], [9, -2, -4, -9, -4], [-2, 3, 3, 10, 9], [6, -3, -4, -7, 1], [-13, -4, -8, 7, 4], [1, -8, 9, -1, 1], [13, -6, 3, -2, 0], [31, -3, -2, -4, -1], [-7, 13, -1, -3, 1], [3, -7, -4, 2, 7], [11, 4, -3, 11, 3], [6, 0, -4, -2, 0], [-2, -2, 12, -10, -4], [17, -5, 2, -1, 8], [4, -5, -3, 0, -3], [31, -6, 4, -8, -5], [17, -10, -9, -5, 5], [19, -5, 8, -2, 3], [8, -11, 10, -11, 1], [20, -8, -6, -9, -3], [3, -16, -1, -1, 3], [15, 7, 5, -2, 0], [15, 4, 5, 3, -3], [-7, 7, -3, -3, -1], [0, 2, -4, 5, -1], [5, 2, 1, 9, -3], [-6, 2, 1, 4, 7], [-7, 12, 1, 5, -7], [-4, 2, -5, 4, 11], [26, 0, -6, 2, 4], [4, -6, 2, 4, 0], [7, 1, 2, -18, -7], [12, -10, -2, 2, -6], [16, 0, 5, -5, -6], [-16, -8, 8, -14, 2], [8, 0, -6, 6, -8], [-8, -2, -7, 4, 5], [1, -10, -3, -1, -3], [4, -9, 6, -3, -3], [-7, 4, -2, 2, 5], [-8, -6, -6, 0, -4], [7, 10, -3, 4, 2], [-3, -2, -5, 13, 11], [1, -8, -7, 1, 9], [13, -8, -11, -3, -1], [4, 8, 9, -4, -7], [-13, 3, -6, 0, 7], [3, 8, -10, 1, 6], [8, -6, -8, 7, -3], [-10, 17, 0, -3, 5], [14, 9, -7, 8, 1], [6, -4, 9, 5, 4], [5, 4, -9, 15, 11], [7, 2, 4, 12, 11], [3, 4, -4, -14, -7], [23, 0, 7, -15, -5], [-1, 14, 1, 8, -2], [0, -6, -2, 10, 0], [-26, 9, -7, 12, 1], [0, -1, -1, 2, -5], [14, 2, -9, -8, 1], [-15, -12, -1, -1, -1], [-5, 0, -6, -7, 0], [-2, 20, -6, 5, -7], [-8, 1, -8, -1, 1], [8, -18, -5, -3, 4], [-12, -1, -1, 0, -7], [-29, 2, -4, -1, -8], [-13, -2, 9, -5, 7], [-2, 9, -1, 4, -1], [-15, 18, 9, 4, 4], [1, -8, -1, -2, 12], [-13, -1, 15, -6, 0], [17, 6, -7, 9, -1], [-33, 18, -3, 17, 7], [9, -4, -7, -6, -2], [0, -4, 10, -8, -12], [27, -7, -3, 2, -6], [4, -3, 7, 6, -5], [12, -6, -4, 4, -10], [-15, 2, -13, -1, -11], [23, -14, 7, -5, 3], [16, -8, 2, -2, 2], [-3, -7, 3, -5, -1], [7, -4, 9, -6, -12], [-17, 10, 6, 0, 1], [-8, 2, 12, -14, 2], [-25, -5, -7, 14, 10], [-4, 6, 14, -10, -6], [-5, -9, 5, -18, 0], [-24, 18, -4, 12, 4], [24, 0, 1, -8, 1], [-17, 10, -5, 9, 5], [16, -7, 15, -22, -5], [20, -6, 2, -2, 4], [32, 0, 4, -1, -3], [14, -6, -21, 6, -3], [1, 8, -7, 1, 9], [-10, -6, -4, 2, 10], [-12, 3, -7, 5, -4], [17, -7, 4, 3, 6], [-17, 4, 7, -3, 1], [-7, 6, 16, -6, 7], [5, 5, 4, 12, 3], [-10, 1, 0, -3, -19], [-6, 12, 1, -6, -7], [3, -2, -3, 8, 6], [42, -12, -5, -11, -2], [-15, 20, -11, 13, 1], [2, -1, 7, -11, 2], [15, -2, 3, 7, 17], [3, 1, -7, 10, -2], [33, 0, -15, 3, 11], [-2, -14, 2, -12, 8], [8, 8, 14, -8, 2], [-1, -18, -6, 3, 8], [12, 13, 11, 2, 7], [-6, -11, -5, 0, 3], [4, 14, 0, -2, -10], [-3, 6, 4, 8, -3], [48, -22, 2, -10, -8], [2, -1, -14, 1, -5], [-19, -5, 4, -17, -10], [6, -5, 6, 2, -8], [-30, 13, 3, 6, 5], [26, 6, -15, -2, 5], [-14, -9, -12, 0, 0], [-28, -7, 17, -2, -5], [20, 2, 4, 0, -4], [-2, -10, 6, -10, 4], [-18, 10, -2, 6, 16], [11, 5, 13, -21, -7], [-17, 12, -7, 9, 5], [14, -4, -13, 10, 7], [-41, 15, 4, 5, 6], [26, -4, 12, -5, 1], [4, -10, -4, 0, 14], [10, -1, 9, -8, 3], [-21, -4, 1, -13, 1], [-2, 9, 1, -10, -5], [-18, 6, 0, 3, 15], [3, 16, 3, 1, -9], [-20, -8, 6, -9, -13], [-23, 5, -11, 15, 13], [-3, -6, 8, -19, -8], [-4, -9, 3, -13, -2], [-23, -5, 9, -8, -12], [-7, 6, 9, 0, -12], [-15, 2, 8, -3, -10], [2, -4, 1, 19, 8], [-6, 6, -6, 5, 5], [-8, 5, -9, 16, 5], [-4, 2, 9, -10, -13], [20, -8, 3, 9, 2], [-10, 4, -20, 20, 4], [-27, 9, -11, 4, -2], [10, -9, 3, -3, -6], [24, -8, -2, -14, -8], [6, 1, -16, 17, -3], [-26, 9, -2, 16, 14], [0, -8, -4, 17, 3], [10, -8, 3, 0, -5], [4, 13, -5, 8, -3], [-6, 8, 16, 0, -8], [13, -13, -3, -8, -18], [3, 6, -9, 17, 13], [3, 10, -18, 17, 16], [36, -5, -1, -3, -12], [-18, -12, 4, -9, -1], [2, -3, 10, 1, -9], [11, -14, 3, -7, 1], [-5, -6, 0, 4, 3], [8, -14, 11, -10, 3], [8, -14, 0, 0, -2], [-36, -6, -2, -11, 5], [-6, 8, 4, 10, 4], [22, 2, -3, -12, -5], [-5, 10, -6, -7, -8], [3, 7, -17, 11, 13], [-14, 0, 1, 3, -8], [-1, 6, -11, 13, 17], [14, 12, 4, -12, -2], [-4, -3, -15, 6, 1], [-34, -6, -2, -6, 10], [-5, 0, -13, 4, 4], [2, -20, -10, -2, 2], [13, 7, -5, 20, 2], [-15, -10, 2, -19, 2], [-42, 5, 0, 14, 2], [4, -6, -3, 8, 9], [11, -12, -6, -5, 10], [1, -14, 9, -1, 1], [-18, -3, -3, -12, -7], [47, -12, 6, -6, -5], [-46, -3, -4, 4, 8], [-18, -6, -8, 2, 12], [44, -1, 4, -1, -7], [-2, -14, -14, -10, 2], [13, -3, -9, -3, -3], [0, 2, -13, 2, -7], [0, 8, -5, 1, 4], [-24, -6, 2, 18, 4], [-12, 9, -6, 12, -8], [-16, 9, 7, -3, -12], [-11, 18, 9, 0, -16], [-5, -1, 7, -4, 0], [-12, -4, 13, -20, -7], [2, 22, -2, -10, 2], [23, -16, 3, 1, 9], [43, -2, -13, -4, -4], [4, 21, -7, -6, 5], [-40, -2, 0, -13, -1], [-25, 1, -11, 8, 10], [-47, 15, -3, 15, 5], [-6, 9, -3, 9, -10], [-23, -14, 23, -17, -5], [-7, -4, 2, -2, -15], [-6, 10, 0, 4, 14], [18, -8, 3, 20, 5], [8, -1, -7, 0, 7], [2, 2, 0, 22, 10], [-6, -4, -10, 0, 0], [-4, -3, -7, -20, -13], [-35, -4, 4, -4, 5], [-34, -14, -14, 9, 9], [-39, 7, -7, 13, 9], [4, -2, -6, 16, 4], [-16, -10, 2, -6, 2], [-46, 0, 10, -6, 2], [33, 8, -14, 9, 14], [24, -6, 4, 6, 10], [41, 7, 5, -11, -11], [-21, 13, 10, 9, -8], [-13, -5, 9, 1, -15], [-3, 4, -9, 35, 13], [-25, 10, 4, 1, -10], [-11, -13, -10, -9, -4], [-24, -8, 7, -1, -14], [-13, -11, 17, -29, -5], [-8, -11, 13, -19, -4], [29, 10, 0, -3, 4], [-5, -8, -13, -4, -16], [12, 8, -1, -4, -9], [33, -12, -5, 1, -9], [0, -4, -1, -12, -13], [8, -11, 3, -2, -3], [-18, 2, 15, -9, -8], [7, 4, 7, 0, -4], [57, -6, -3, 3, -1], [-24, 16, 5, 1, 8], [22, -11, -14, -2, -4], [-6, 9, 3, 18, 5], [-26, 15, 2, 12, 10], [10, 4, 0, 4, 12], [10, 1, 16, -6, -2], [20, -16, 20, -26, -12], [-25, 11, -6, -14, -11], [20, -5, -2, -5, 7], [16, -9, 12, -17, -7], [-28, -4, 8, -10, 8], [20, 9, -1, -2, 3], [-28, 10, -8, 20, 4], [11, -10, 6, 9, 12], [-9, -2, 25, -17, -1], [25, -19, -2, -20, 1], [47, 6, 2, 7, -14], [-38, 18, -2, 4, -4], [-29, -6, -10, -15, -12], [-26, -13, 16, -10, 6], [-11, 3, 19, -22, 0], [12, -19, 2, -2, -6], [24, 26, -3, -3, -6], [-30, 24, 22, 15, 5], [-52, 17, -10, 18, 10], [17, -13, 3, 3, 11], [-26, -8, -8, 24, 8], [-11, 15, -3, -3, -17], [-19, -7, 1, 9, 7], [26, -8, 7, 2, 11], [-43, -3, 1, 7, 9], [-6, 3, 7, -24, -13], [-18, 4, -12, 16, 16], [0, 2, 0, 2, 10], [25, -12, -9, -16, -10], [5, 4, -3, -7, 3], [-10, -4, -5, -16, -15], [9, 4, -11, -1, -3], [-22, -9, 8, -20, 8], [51, -2, 3, 7, 7], [-20, 18, -9, 19, 20], [-44, 16, -11, 30, 17], [-12, -15, 9, -4, 5], [-20, 20, -18, 10, -8], [8, -10, -4, 0, 10], [-8, 19, 1, 8, -1], [-7, 8, 10, 10, -3], [11, 16, -4, -9, -8], [-19, 12, -5, -5, -9], [6, -3, -4, -10, 4], [26, 8, -3, 9, 6], [29, 6, 3, -6, -8], [-2, 22, -25, 21, -2], [-19, 10, 14, 3, -8], [-12, 3, 4, -30, -8], [-2, -6, 21, -28, -21], [-35, 11, 5, 2, 6], [-6, -19, -10, -3, 19], [-17, -5, -20, 0, -3], [-6, 15, -9, -5, -10], [-22, -16, -16, -8, -4], [11, 6, 11, -11, -13], [19, 14, -16, 13, 2], [16, 10, -20, 6, 10], [8, 14, -1, 11, -4], [29, -5, 9, -1, -3], [-20, -1, 9, 3, 4], [26, -14, 26, -30, -6], [42, 13, 5, 4, -3], [7, 5, -25, 9, 5], [13, -11, -8, 11, 10], [-48, 10, 1, -5, 2], [-66, 9, 0, 6, -2], [18, 8, 0, 19, 9], [-16, 24, 8, 2, 16], [43, 8, -8, 9, -2], [20, 8, -5, -2, 13], [15, -6, 9, -1, -7], [-12, -23, -3, -3, -2], [34, -7, -1, -10, 1], [-9, -9, 13, -11, -15], [-2, 10, -26, 18, 6], [-1, 9, -13, -1, -3], [26, -17, -4, -24, -16], [22, -4, -14, 24, 4], [18, -20, 14, -14, -10], [84, 2, -6, 2, 0], [-7, 6, 15, -14, -8], [-17, 6, 22, -6, 7], [17, -10, 1, -18, -10], [-10, 24, 3, -1, 0], [44, -4, -1, -8, -5], [2, -18, -4, -20, -2], [-14, 4, 5, 9, 8], [47, 10, -7, -7, -5], [-9, 6, -5, 23, 19], [-16, -14, 18, -10, 0], [-1, -7, -5, 25, 7], [-20, 30, -7, 5, 16], [-18, 15, 3, -17, -8], [8, 24, -10, 8, -14], [-14, -20, -6, -2, -10], [-6, 8, 10, 4, 20], [-29, 22, 13, -9, 13], [-20, -29, 3, 17, 4], [-27, -4, -1, -2, 8], [-27, 18, -9, -1, 3]]
